@@ -42,7 +42,7 @@ generic_interface_init (gpointer iface, gpointer data)
 		/* callback_info takes over ownership of perl_method_name. */
 		callback_info = create_perl_callback_closure_for_named_sub (
 		                  field_interface_info, perl_method_name);
-		dwarn ("generic_interface_init: installing vfunc %s.%s as %s at offset %d (vs. %d) inside %p\n",
+		dwarn ("installing vfunc %s.%s as %s at offset %d (vs. %d) inside %p\n",
 		       g_base_info_get_name (info), vfunc_name, perl_method_name,
 		       field_offset, g_vfunc_info_get_offset (vfunc_info),
 		       iface);
@@ -61,6 +61,6 @@ generic_interface_finalize (gpointer iface, gpointer data)
 {
 	GIInterfaceInfo *info = data;
 	PERL_UNUSED_VAR (iface);
-	dwarn ("releasing interface info\n");
+	dwarn ("info = %p\n", info);
 	g_base_info_unref ((GIBaseInfo *) info);
 }
